package clients

import (
    "encoding/json"
    "fmt"
    "io/ioutil"
    "net/http"
    "strings"
    "time"

    "discord-cfb-bot/config"
)

type Game struct {
    ID           int         `json:"id"`
    StartDate    string      `json:"startDate"`
    Status       string      `json:"status"`
    Period       *int        `json:"period"`
    Clock        *string     `json:"clock"`
    HomeTeam     Team        `json:"homeTeam"`
    AwayTeam     Team        `json:"awayTeam"`
}

type Team struct {
    Name   string `json:"name"`
    Points *int   `json:"points"`
}

func GetGameInfo(teamName string) string {
    apiURL := "https://apinext.collegefootballdata.com/scoreboard"
    req, _ := http.NewRequest("GET", apiURL, nil)
    req.Header.Set("Authorization", "Bearer "+config.CFBDAPIKey) // Use the API key from config

    client := &http.Client{}
    resp, err := client.Do(req)
    if err != nil {
        return fmt.Sprintf("Failed to reach the scoreboard API: %v", err)
    }
    defer resp.Body.Close()

    if resp.StatusCode != http.StatusOK {
        return fmt.Sprintf("Failed to reach the scoreboard API. Status code: %d", resp.StatusCode)
    }

    body, _ := ioutil.ReadAll(resp.Body)

    var games []Game
    if err := json.Unmarshal(body, &games); err != nil {
        return "Error parsing the API response."
    }

    for _, game := range games {
        if strings.Contains(strings.ToLower(game.HomeTeam.Name), strings.ToLower(teamName)) ||
            strings.Contains(strings.ToLower(game.AwayTeam.Name), strings.ToLower(teamName)) {

            startTime, _ := time.Parse(time.RFC3339, game.StartDate)
            adjustedTime := startTime.Add(-5 * time.Hour) // Subtract 5 hours for upcoming games
            formattedTime := adjustedTime.Format("03:04 PM")

            if game.Status == "in_progress" {
                return fmt.Sprintf("%s: **%d** %s: **%d** | Quarter: %d | Time Remaining: %s",
                    game.AwayTeam.Name, valueOrZero(game.AwayTeam.Points),
                    game.HomeTeam.Name, valueOrZero(game.HomeTeam.Points),
                    *game.Period, derefOrDefault(game.Clock, "00:00"))
            } else if game.Status == "scheduled" {
                return fmt.Sprintf("Upcoming: %s @ %s at %s Eastern",
                    game.AwayTeam.Name, game.HomeTeam.Name, formattedTime)
            } else if game.Status == "completed" {
                return fmt.Sprintf("Final: %s: %d %s: %d",
                    game.AwayTeam.Name, valueOrZero(game.AwayTeam.Points),
                    game.HomeTeam.Name, valueOrZero(game.HomeTeam.Points))
            }
        }
    }

    return "No game found for the specified team."
}

// Helper function to safely dereference pointers
func derefOrDefault(str *string, defaultVal string) string {
    if str != nil {
        return *str
    }
    return defaultVal
}

// Helper function to return a value or zero if nil
func valueOrZero(val *int) int {
    if val != nil {
        return *val
    }
    return 0
}
